--------------------------------------------------------------------------------
-- LAB4 prototype: 9-bit arithmetic and logic unit 
-- Plan C2: this is the direct translation into VHDL of the schematic (Fig. 3) 
-- available at: 
-- https://digsys.upc.edu/csd/P04/lab04/lab4.html#Prot
---------------------------------------------------
-- Project P4  - CSD 
-- http://digsys.upc.edu
--------------------------------------------------------------------------------
LIBRARY ieee;
USE  IEEE.STD_LOGIC_1164.all;

ENTITY ALU_9bit IS
	PORT (
		A		: IN 	STD_LOGIC_VECTOR(8 DOWNTO 0);-- DE10-Lite switches 8..0	
		B		: IN 	STD_LOGIC_VECTOR(8 DOWNTO 0); -- PCB switches
		OP		: IN 	STD_LOGIC_VECTOR(2 DOWNTO 0); -- PCB switches    
		R		: OUT 	STD_LOGIC_VECTOR (17 DOWNTO 0); -- PCB LED (active-high)
											-- Flag indicators
		C, N, OV, Z, EQ	: OUT  	STD_LOGIC; 	-- PCB LED (active-high)	
									-- DE10-Lite 7-segment digits (active-low)
		HEX4_L, HEX3_L			: OUT 	STD_LOGIC_VECTOR (6 DOWNTO 0);	
		HEX2_L, HEX1_L, HEX0_L	: OUT 	STD_LOGIC_VECTOR (6 DOWNTO 0);
		
		MS_LED	: OUT  	STD_LOGIC -- Minus sign --> segment HEX5_L(6)

	  );
END ALU_9bit;

ARCHITECTURE hierarchical_plan_C2 OF ALU_9bit IS

-- The component to be used:
COMPONENT Nonuple_MUX_4 IS
	PORT (
		Ch0, Ch1, Ch2, Ch3	: 	IN 	STD_LOGIC_VECTOR (8 downto 0);  
		S					:	IN 	STD_LOGIC_VECTOR (1 downto 0);	
		E					: 	IN 	STD_LOGIC;
		Y					: 	OUT	STD_LOGIC_VECTOR (8 downto 0)
	  );
END COMPONENT;

COMPONENT Nonuple_MUX_2 IS
	PORT (
		Ch0, Ch1	: 	IN 	STD_LOGIC_VECTOR (8 downto 0);  
		S			:	IN 	STD_LOGIC; 
		E			: 	IN 	STD_LOGIC;
		Y			: 	OUT	STD_LOGIC_VECTOR (8 downto 0)
	  );
END COMPONENT;


COMPONENT Quad_MUX_2 IS
	PORT (
		Ch0, Ch1	: 	IN 	STD_LOGIC_VECTOR (3 downto 0);  
		S			:	IN 	STD_LOGIC; 
		E			: 	IN 	STD_LOGIC;
		Y			: 	OUT	STD_LOGIC_VECTOR (3 downto 0)
	  );
END COMPONENT;

COMPONENT Sexdecuple_MUX_2 IS
	PORT (
		Ch0, Ch1	: 	IN 	STD_LOGIC_VECTOR (15 downto 0);  
		S			:	IN 	STD_LOGIC; 
		E			: 	IN 	STD_LOGIC;
		Y			: 	OUT	STD_LOGIC_VECTOR (15 downto 0)
	  );
END COMPONENT;

COMPONENT MUX_2 IS
	PORT (							
		Ch0, Ch1	: 	IN 	STD_LOGIC;  
		S			:	IN 	STD_LOGIC; 
		E			: 	IN 	STD_LOGIC;
		Y			:	OUT	STD_LOGIC
	  );
END COMPONENT;

COMPONENT Adder_9bit IS
	PORT (
		A,B		: IN 	STD_LOGIC_VECTOR(8 DOWNTO 0); 				
		Cin		: IN 	STD_LOGIC;    
		S		: OUT 	STD_LOGIC_VECTOR (8 DOWNTO 0);	
		Cout	: OUT  	STD_LOGIC;
		C8	 	: OUT  	STD_LOGIC --Auxiliary carry out for overflow calculation
	  );
END COMPONENT;

COMPONENT Int_Mult_9bit IS
   port
   (
      A, B:	in	std_logic_vector(8 downto 0);
      P, Q:	out	std_logic_vector(17 downto 0)
   );
END COMPONENT;

COMPONENT Two_C_9bit IS
	PORT (
		A		: IN 	STD_LOGIC_VECTOR(8 DOWNTO 0); 				
		B		: OUT 	STD_LOGIC_VECTOR (8 DOWNTO 0)
	  );
END COMPONENT;

COMPONENT Dec_hex_7seg IS
	PORT (
		D					: IN STD_LOGIC_VECTOR(3 DOWNTO 0); 
		BI_L, LT_L, RBI_L	: IN STD_LOGIC;      
		RBO_L				: OUT STD_LOGIC; 		
		Y_L					: OUT STD_LOGIC_VECTOR(6 DOWNTO 0)
		);
END COMPONENT;

COMPONENT Bin_BCD_16bit IS
	port (
		B					: 	in	STD_LOGIC_VECTOR(15 DOWNTO 0); 
		TTH, TH, H, T, U	: 	out	STD_LOGIC_VECTOR(3 DOWNTO 0)
	);

END COMPONENT;



--------------------------------------------------------------------------------
-- Internal signals
	SIGNAL	D, E, F, G, H		: STD_LOGIC_VECTOR(8 downto 0);	
	SIGNAL	J, K, L, P, Q, MR	: STD_LOGIC_VECTOR(8 downto 0);	
	SIGNAL	M					: STD_LOGIC_VECTOR(17 downto 0);	
	SIGNAL	S 					: STD_LOGIC_VECTOR(1 downto 0);	
	SIGNAL	C8, Cout, V, W, T	: STD_LOGIC;	
	
	SIGNAL	CF, OVF, ZF, EQF, NF: STD_LOGIC;

	SIGNAL	AP	: STD_LOGIC_VECTOR(8 downto 0);	-- Arithmetic absolute value
	SIGNAL	AR	: STD_LOGIC_VECTOR(15 downto 0);-- Result absolute value	
	SIGNAL	Y	: STD_LOGIC_VECTOR(17 downto 0); --Multiplier absolute value
	SIGNAL	TTHD, THD, HD, TD, UD	: STD_LOGIC_VECTOR(3 downto 0);--BCD digits

	SIGNAL	RB 	: STD_LOGIC_VECTOR(4 downto 0);-- Ripple blanking chain 

	SIGNAL	OVFA, BI_L 	: STD_LOGIC; -- Used to show "E" in digit HEX0 when OV
	
			-- Units digit or error symbol ("E") 
	SIGNAL	UDE			: STD_LOGIC_VECTOR(3 downto 0); 

BEGIN
-- Instantiating the carry generator 
Chip1	: Nonuple_MUX_4
	PORT MAP (
-- from component	=> to signal or port name
		E		=> '1',			
		Ch0		=> D,
		Ch1		=> E,
		Ch2		=> F,
		Ch3		=> G,
		S		=> OP(1 downto 0), 
		Y		=> H
	);  
			
Chip2	: Nonuple_MUX_2
	PORT MAP (
-- from component	=> to signal or port name
		E		=> '1',			
		Ch0		=> A,
		Ch1		=> "000000000",
		S		=> S(0), 
		Y		=> J
	); 

Chip3	: Adder_9bit
	PORT MAP (
-- from component	=> to signal or port name
		Cin		=> S(1),			
		A		=> J,
		B		=> K,
		Cout	=> Cout, 
		C8		=> C8, 
		S		=> L
	); 

Chip4	: Int_Mult_9bit
	PORT MAP (
-- from component	=> to signal or port name
		A		=> A,			
		B		=> B,
		P		=> M,
		Q 		=> Y  -- The positive multiplication result
	); 				  -- will be used for BCD conversion and display

Chip5	: Nonuple_MUX_2
	PORT MAP (
-- from component	=> to signal or port name
		E		=> '1',			
		Ch0		=> H,
		Ch1		=> L,
		S		=> OP(2), 
		Y		=> P
	); 

Chip6	: Nonuple_MUX_2
	PORT MAP (
-- from component	=> to signal or port name
		E		=> '1',			
		Ch0		=> P,
		Ch1		=> M(8 downto 0),
		S		=> W, 
		Y		=> Q
	); 

Chip7	: MUX_2
	PORT MAP (
-- from component	=> to signal or port name
		E		=> '1',			
		Ch0		=> T,
		Ch1		=> V,
		S		=> W, 
		Y		=> ZF
	); 
	
Chip8	: MUX_2
	PORT MAP (
-- from component	=> to signal or port name
		E		=> '1',			
		Ch0		=> Q(8),
		Ch1		=> M(17),
		S		=> W, 
		Y		=> NF
	); 

Chip9	: Two_C_9bit
	PORT MAP (
-- from component	=> to signal or port name
		A		=> P,			
		B		=> AP
	); 

Chip10	: Sexdecuple_MUX_2
	PORT MAP (
-- from component	=> to signal or port name
		E		=> '1',			
		Ch0(15 downto 9)	=> "0000000",
		Ch0(8 downto 0) 	=> AP,		
		Ch1					=> Y(15 downto 0),
		S					=> W, 
		Y					=> AR
	); 

Chip11	: Bin_BCD_16bit
	PORT MAP (
-- from component	=> to signal or port name
		B		=> AR, 			
		TTH		=> TTHD,
		TH		=> THD,		
		H		=> HD,
		T		=> TD, 
		U		=> UD
	); 

-- The six BCD to seven segment converters
-- Used only for displaying arithmetic results
Chip12	: Dec_hex_7seg
	PORT MAP (
-- from component	=> to signal or port name
		D		=> TTHD, 			
		BI_L	=> BI_L,
		LT_L	=> '1',		
		RBI_L	=> '0', -- initialise the zero blanking chain
		RBO_L	=> RB(4), 
		Y_L		=> HEX4_L(6 downto 0)
	); 

Chip13	: Dec_hex_7seg
	PORT MAP (
-- from component	=> to signal or port name
		D		=> THD, 			
		BI_L	=> BI_L,
		LT_L	=> '1',		
		RBI_L	=> RB(4), 
		RBO_L	=> RB(3), 
		Y_L		=> HEX3_L(6 downto 0)
	); 

Chip14	: Dec_hex_7seg
	PORT MAP (
-- from component	=> to signal or port name
		D		=> HD, 			
		BI_L	=> BI_L,
		LT_L	=> '1',		
		RBI_L	=> RB(3), 
		RBO_L	=> RB(2), 
		Y_L		=> HEX2_L(6 downto 0)
	); 

Chip15	: Dec_hex_7seg
	PORT MAP (
-- from component	=> to signal or port name
		D		=> TD, 			
		BI_L	=> BI_L,
		LT_L	=> '1',		
		RBI_L	=> RB(2), 
		RBO_L	=> RB(1), 
		Y_L		=> HEX1_L(6 downto 0)
	); 

Chip16	: Dec_hex_7seg
	PORT MAP (
-- from component	=> to signal or port name
		D		=> UDE, 	-- Units digit or Error symbol		
		BI_L	=> OP(2), 
		LT_L	=> '1',		-- not used 
		RBI_L	=> '1',		-- not chained 
		RBO_L	=> RB(0), 
		Y_L		=> HEX0_L(6 downto 0)
	); 


Chip17	: Quad_MUX_2
	PORT MAP (
-- from component	=> to signal or port name
		E		=> '1',			
		Ch0		=> UD,
		Ch1		=> "1110", -- Symbol hexadecimal "E" (error)
		S		=> OVFA, 
		Y		=> UDE
	); 


-- Gate blocks 
D <= A or B; --  this is a block of 9 OR gates

E <= A and B;

F <= A xor B;

G <= not(A);

K(0) <= B(0) xor S(1);
K(1) <= B(1) xor S(1);
K(2) <= B(2) xor S(1);
K(3) <= B(3) xor S(1);
K(4) <= B(4) xor S(1);
K(5) <= B(5) xor S(1);
K(6) <= B(6) xor S(1);
K(7) <= B(7) xor S(1);
K(8) <= B(8) xor S(1);

-- Overflow flag for integer operations:
OVF <= Cout xor C8;

-- Carry out for radix-2 additions:  
CF <= Cout; 

-- Equal operators detector flag (EQ) 
EQF <= not(F(0) or F(1) or F(2) or F(3) or F(4) or F(5) or F(6) or F(7) or F(8));

-- Zero detector for 7 operations:  
T <= not(P(0) or P(1) or P(2) or P(3) or P(4) or P(5) or P(6) or P(7) or P(8));

--------------------------------------------------------------------------------
-- Zero detector for the multiplication:  
V <= not(M(0) or M(1) or M(2) or M(3) or M(4) or M(5) or M(6) or M(7) or M(8) or
         M(9) or M(10) or M(11) or M(12) or M(13) or M(14) or M(15) or 
         M(16) or M(17));

-- Multiplication selector 
W <= OP(2) and OP(1) and OP(0);

-- Adder and subtractor selector
S(0) <= OP(1); 
S(1) <= OP(1) or OP(0);

-- Result buffer
R (8 downto 0) <= Q; 
R (17 downto 9) <= MR(8 downto 0); 


-- AND block to disable the Multiplier result when not selected 
MR(0) <= M(9)  and W;
MR(1) <= M(10) and W;
MR(2) <= M(11) and W;
MR(3) <= M(12) and W;
MR(4) <= M(13) and W;
MR(5) <= M(14) and W;
MR(6) <= M(15) and W;
MR(7) <= M(16) and W;
MR(8) <= M(17) and W;

-- To show "E" in the HEX0_L when overflow 
OVFA 	<= OVF and OP(2) and not(OP(1)); -- only ADD, SUBT, 2C(B)

-- Blanking when logic operations or when OV in arithmetic operations
BI_L 	<= not(OVFA) and OP(2); 


-- Enable carries logic depending on the type of operation
EQ 	<= EQF; -- All operations
Z 	<= ZF; -- All operations
C 	<= CF and OP(2) and not(OP(1)) and not(OP(0)); -- only ADD 
OV	<= OVFA; -- only ADD, SUBT, 2C(B)
N 	<= NF and OP(2); -- only arithmetic 


-- Negative sign LED in the HEX5_L
MS_LED <= OVFA or not(NF) or not(OP(2));
-- 

END hierarchical_plan_C2;