--------------------------------------------------------------------------------
-- LAB4 prototype: 9-bit arithmetic and logic unit 
-- Plan C2: this is the direct translation into VHDL of the schematic (Fig. 3) 
-- available at: 
-- https://digsys.upc.edu/csd/P04/lab04/lab4.html#Prot
---------------------------------------------------
-- Project P4  - CSD 
-- http://digsys.upc.edu
--------------------------------------------------------------------------------
LIBRARY ieee;
USE  IEEE.STD_LOGIC_1164.all;

ENTITY ALU_9bit_top IS
	PORT (
		A		: IN 	STD_LOGIC_VECTOR(8 DOWNTO 0);-- DE10-Lite switches 8..0	
		B		: IN 	STD_LOGIC_VECTOR(8 DOWNTO 0); -- PCB switches
		OP		: IN 	STD_LOGIC_VECTOR(2 DOWNTO 0); -- PCB switches    
		R		: OUT 	STD_LOGIC_VECTOR (15 DOWNTO 0); -- PCB LED (active-high)
											-- Flag indicators
		C, N, OV, Z, EQ	: OUT  	STD_LOGIC; 	-- PCB LED (active-high)	
				-- DE10-Lite 7-segment digits (active-low)
		HEX5_L, HEX4_L, HEX3_L: OUT 	STD_LOGIC_VECTOR (7 DOWNTO 0);	
		HEX2_L, HEX1_L, HEX0_L: OUT 	STD_LOGIC_VECTOR (7 DOWNTO 0);

			-- DE10-Lite LED array (active-high) not used 
		LED	  : OUT 	STD_LOGIC_VECTOR (9 DOWNTO 0)
	  );
END ALU_9bit_top;

ARCHITECTURE hierarchical_plan_C2 OF ALU_9bit_top IS

-- The component to be used:
COMPONENT ALU_9bit IS
	PORT (
		A		: IN 	STD_LOGIC_VECTOR(8 DOWNTO 0);-- DE10-Lite switches 8..0	
		B		: IN 	STD_LOGIC_VECTOR(8 DOWNTO 0); -- PCB switches
		OP		: IN 	STD_LOGIC_VECTOR(2 DOWNTO 0); -- PCB switches    
		R		: OUT 	STD_LOGIC_VECTOR (17 DOWNTO 0); -- PCB LED (active-high)
											-- Flag indicators
		C, N, OV, Z, EQ	: OUT  	STD_LOGIC; 	-- PCB LED (active-high)	
									-- 7-segment digits (active-low)
		HEX4_L, HEX3_L			: OUT 	STD_LOGIC_VECTOR (6 DOWNTO 0);	
		HEX2_L, HEX1_L, HEX0_L	: OUT 	STD_LOGIC_VECTOR (6 DOWNTO 0);
		
		MS_LED	: OUT  	STD_LOGIC 	-- Minus sign --> segment HEX5_L(6)

	);
END COMPONENT;



--------------------------------------------------------------------------------
-- Internal signals
-- The two most significant output results from the signed multiplier not used  
	SIGNAL	K : STD_LOGIC_VECTOR(17 downto 16);	

BEGIN
-- Instantiating the carry generator 
Chip1	: ALU_9bit
	PORT MAP (
-- from component		=> to signal or port name
		A				=>	A, 
		B				=>	B, 
		OP				=>	OP,  
		R(15 downto 0)	=>	R, 
		R(17 downto 16)	=>	K, 		
						-- Flag indicators
		C				=>	C, 
		N				=>	N, 
		OV				=>	OV, 
		Z				=>	Z, 
		EQ				=>	EQ, 
			--		DE10-Lite 7-segment digits (active-low)
		HEX4_L			=>	HEX4_L(6 downto 0), 
		HEX3_L			=>	HEX3_L(6 downto 0),			
		HEX2_L			=>	HEX2_L(6 downto 0), 
		HEX1_L			=>	HEX1_L(6 downto 0),
		HEX0_L			=>	HEX0_L(6 downto 0), 
				
		MS_LED			=>	HEX5_L(6)
	);  
			


-- HEX5 LED not used: 
HEX5_L(5 downto 0 ) <= "111111"; 

-- Decimal points not used:
HEX5_L(7) <= '1'; 
HEX4_L(7) <= '1'; 
HEX3_L(7) <= '1'; 
HEX2_L(7) <= '1'; 
HEX1_L(7) <= '1'; 
HEX0_L(7) <= '1'; 

-- Board DE10-Lite LED not used 
LED(9 downto 0) <= "0000000000"; 

-- 

END hierarchical_plan_C2;