--------------------------------------------------------------------------------
-- An example of a 4-bit full carry-lookahead adder using a hierarchical 
-- structure of components.
-- Plan C2: this is the direct translation into VHDL of the schematic (Fig. 8) 
-- available at: 
-- https://digsys.upc.edu/csd/P03/Adder_4bit_CLA/Adder_4bit.html
---------------------------------------------------
-- Exercise P3  - CSD 
-- http://digsys.upc.edu
--------------------------------------------------------------------------------
LIBRARY ieee;
USE  IEEE.STD_LOGIC_1164.all;

ENTITY Adder_4bit IS
	PORT (
		A,B		: IN 	STD_LOGIC_VECTOR(3 DOWNTO 0); 				
		Cin		: IN 	STD_LOGIC;    
		S		: OUT 	STD_LOGIC_VECTOR (3 DOWNTO 0);	
		Cout	: OUT  	STD_LOGIC;
		P, G 	: OUT  	STD_LOGIC
	  );

END Adder_4bit;

ARCHITECTURE hierarchical_plan_C2 OF Adder_4bit IS


-- The component to be used:
COMPONENT Carry_generator IS
	PORT (
		Cin				: IN STD_LOGIC; 
		P0, P1, P2, P3	: IN STD_LOGIC; 
		G0, G1, G2, G3	: IN STD_LOGIC;
		Cout, C3, C2, C1	: OUT STD_LOGIC;
		P, G			: OUT STD_LOGIC
	  );
END COMPONENT;

-- Internal signals
	SIGNAL	P0, P1, P2, P3 	: STD_LOGIC;	
	SIGNAL	G0, G1, G2, G3 	: STD_LOGIC;	
	SIGNAL	C1, C2, C3		: STD_LOGIC;
BEGIN
-- Instantiating the carry generator 
chip_1	: Carry_generator
	PORT MAP (
-- from component	=> to signal or port name
		Cin			=> Cin,			
		P0			=> P0,
		G0			=> G0,
		P1			=> P1,
		G1			=> G1,
		P2			=> P2,
		G2			=> G2,
		P3			=> P3,
		G3			=> G3,
		C1			=> C1,
		C2			=> C2,
		C3			=> C3,
		P			=> P,
		G			=> G,		
		Cout		=> Cout	
	);  
				
-- Other circuits and equations:
-- sum outputs: 
S(0) <= P0 xor Cin;
S(1) <= P1 xor C1;
S(2) <= P2 xor C2;
S(3) <= P3 xor C3;
-- Propagators
P0 <= A(0) xor B(0);
P1 <= A(1) xor B(1);
P2 <= A(2) xor B(2);
P3 <= A(3) xor B(3);
-- Generators
G0 <= A(0) and B(0);
G1 <= A(1) and B(1);
G2 <= A(2) and B(2);
G3 <= A(3) and B(3);

END hierarchical_plan_C2;