--------------------------------------------------------------------------------
-- An example of 9-bit full carry-lookahead adder using a hierarchical 
-- structure of components.
-- Plan C2: this is the adaptation of the the schematic 
-- available at: 
-- https://digsys.upc.edu/csd/P04/lab04/lab4.html#Adder_16bit_CLA
---------------------------------------------------
-- Exercise P4 -Lab4 - CSD - Arithmetic circuits and circuit's speed 
-- http://digsys.upc.edu
--------------------------------------------------------------------------------
LIBRARY ieee;
USE  IEEE.STD_LOGIC_1164.all;

ENTITY Adder_9bit IS
	PORT (
		A,B		: IN 	STD_LOGIC_VECTOR(8 DOWNTO 0); 				
		Cin		: IN 	STD_LOGIC;    
		S		: OUT 	STD_LOGIC_VECTOR (8 DOWNTO 0);	
		Cout	: OUT  	STD_LOGIC;
		C8		: OUT  	STD_LOGIC
	  );

END Adder_9bit;

ARCHITECTURE hierarchical_plan_C2 OF Adder_9bit IS

-- The component to be used:
COMPONENT Carry_generator IS
	PORT (
		Cin					: IN STD_LOGIC; 
		P0, P1, P2, P3		: IN STD_LOGIC; 
		G0, G1, G2, G3		: IN STD_LOGIC;
		Cout, C3, C2, C1	: OUT STD_LOGIC;
		P, G				: OUT STD_LOGIC
	  );
END COMPONENT;

COMPONENT Adder_4bit IS
	PORT (
		A,B		: IN 	STD_LOGIC_VECTOR(3 DOWNTO 0); 				
		Cin		: IN 	STD_LOGIC;    
		S		: OUT 	STD_LOGIC_VECTOR (3 DOWNTO 0);	
		Cout	: OUT  	STD_LOGIC;
		P, G 	: OUT  	STD_LOGIC
	);
END COMPONENT;

COMPONENT Adder_1bit IS
	PORT (
		Ai,Bi, Ci	: IN STD_LOGIC;				
		So, Co		: OUT STD_LOGIC
	);
END COMPONENT;

-- Internal signals
	SIGNAL	P0, P4, P			: STD_LOGIC;	
	SIGNAL	G0, G4, G			: STD_LOGIC;	
	SIGNAL	Cout4, Cout8, Cout12: STD_LOGIC;
	SIGNAL	K4, K8, K12			: STD_LOGIC;
	
BEGIN
-- Instantiating the carry generator 
chip_4	: Carry_generator
	PORT MAP (
-- from component	=> to signal or port name
		Cin			=> Cin,			
		P0			=> P0,
		G0			=> G0,
		P1			=> P4,
		G1			=> G4,
		P2			=> '0',
		G2			=> '0',
		P3			=> '0',
		G3			=> '0',
		C1			=> Cout4,
		C2			=> Cout8,
		C3			=> Cout12,
		P			=> P,
		G			=> G,		
		Cout		=> K12	
	);  

-- Instantiating the 4-bit adders:
chip_1	: Adder_4bit
	PORT MAP (
-- from component	=> to signal or port name
		Cin			=> Cin,			
		A			=> A(3 downto 0),
		B			=> B(3 downto 0),
		S			=> S(3 downto 0),
		P			=> P0,
		G			=> G0,	
		Cout		=> K4 -- not used in this application	
	);

chip_2	: Adder_4bit
	PORT MAP (
-- from component	=> to signal or port name
		Cin			=> Cout4,			
		A			=> A(7 downto 4),
		B			=> B(7 downto 4),
		S			=> S(7 downto 4),
		P			=> P4,
		G			=> G4,	
		Cout		=> K8 -- not used in this application	
	);

chip_3	: Adder_1bit
	PORT MAP (
-- from component	=> to signal or port name
		Ci			=> Cout8,			
		Ai			=> A(8),
		Bi			=> B(8),
		So			=> S(8),
		Co			=> Cout 
	);

			
-- Other circuits and equations:

C8 <= Cout8;

END hierarchical_plan_C2;