-------------------------------------------------------------------------------
-- Example of a hierarchical 16-bit binary radix-2 to BCD converter 
-- Plan C2. 
-- This VHDL is the translation of the schematic in Fig. 2 in the annex A3 
-- of the LAB4 prototype on designing an ALU_9bit.
-- https://digsys.upc.edu/csd/P04/lab04/lab4.html#ProtA3
-- 
-- The component DM74185 is our adaptation using plan B of the classic 
-- chip 74185A truth table. Is is described in the following annex:  
-- https://digsys.upc.edu/csd/P02/Bin_BCD_6bitB/bin_BCD_6bit.html#DM74185

---------------------------------------------------
-- Exercise P3  - CSD - Plan C2 using multiple VHDL files 
-- http://digsys.upc.edu
------------------------------------------------------------------------------

LIBRARY ieee;
USE  IEEE.STD_LOGIC_1164.all;

entity Bin_BCD_16bit is
	port (
		B					: 	in	STD_LOGIC_VECTOR(15 DOWNTO 0); 
		TTH, TH, H, T, U	: 	out	STD_LOGIC_VECTOR(3 DOWNTO 0)
	);

end Bin_BCD_16bit;
	
architecture structural_planC2 of Bin_BCD_16bit is

-- Components to be used:
COMPONENT DM74185 IS  
	port (
		E, D, C, B, A 	: 	in STD_LOGIC;
		Y				: 	out STD_LOGIC_VECTOR(6 DOWNTO 1)
	);
END COMPONENT;

-- Signals (wires and cables) 
	SIGNAL	C, D: STD_LOGIC_VECTOR(5 downto 0); 
	SIGNAL	E, F: STD_LOGIC_VECTOR(11 downto 0); 
	SIGNAL	G	: STD_LOGIC_VECTOR(17 downto 0); 
	SIGNAL	Q	: STD_LOGIC_VECTOR(13 downto 0); 
	SIGNAL	J	: STD_LOGIC_VECTOR(6 downto 0); 
	SIGNAL	K	: STD_LOGIC_VECTOR(1 downto 0); 	
	
BEGIN
-- Component instantiation 
	Chip1	: DM74185
		PORT MAP (
-- from component name 	=> to signal or port name
			E	=> B(15),
			D 	=> B(14),
			C	=> B(13),
			B 	=> B(12),
			A	=> B(11),
			Y	=> C 	
		);	

	Chip2	: DM74185
		PORT MAP (
-- from component name 	=> to signal or port name
			E	=> C(2),
			D 	=> C(1),
			C	=> C(0),
			B 	=> B(10),
			A	=> B(9),
			Y	=> D
		);	

	Chip3	: DM74185
		PORT MAP (
-- from component name 	=> to signal or port name
			E	=> C(5),
			D	=> C(4),
			C	=> C(3),
			B	=> D(4),
			A	=> D(3),
			Y	=> E(11 downto 6)			
		);		

	Chip4	: DM74185
		PORT MAP (
-- from component name 	=> to signal or port name
			E	=> D(2),
			D 	=> D(1),
			C	=> D(0),
			B 	=> B(8),
			A	=> B(7),
			Y	=> E(5 downto 0)
		);

	Chip5	: DM74185
		PORT MAP (
-- from component name 	=> to signal or port name
			E	=> E(8),
			D 	=> E(7),
			C	=> E(6),
			B 	=> E(4),
			A	=> E(3),
			Y	=> F(11 downto 6)
		);

	Chip6	: DM74185
		PORT MAP (
-- from component name 	=> to signal or port name
			E	=> E(2),
			D 	=> E(1),
			C	=> E(0),
			B 	=> B(6),
			A	=> B(5),
			Y	=> F(5 downto 0)
		);

	Chip7	: DM74185
		PORT MAP (
-- from component name 	=> to signal or port name
			E	=> E(11),
			D 	=> E(10),
			C	=> E(9),
			B 	=> F(10),
			A	=> F(9),
			Y	=> G(17 downto 12)
		);

	Chip8	: DM74185
		PORT MAP (
-- from component name 	=> to signal or port name
			E	=> F(8),
			D 	=> F(7),
			C	=> F(6),
			B 	=> F(4),
			A	=> F(3),
			Y	=> G(11 downto 6)
		);

	Chip9	: DM74185
		PORT MAP (
-- from component name 	=> to signal or port name
			E	=> F(2),
			D 	=> F(1),
			C	=> F(0),
			B 	=> B(4),
			A	=> B(3),
			Y	=> G(5 downto 0)
		);
		
	Chip10	: DM74185
		PORT MAP (
-- from component name 	=> to signal or port name
			E	=> G(14),
			D 	=> G(13),
			C	=> G(12),
			B 	=> G(10),
			A	=> G(9),
			Y	=> Q(13 downto 8)
		);

	Chip11	: DM74185
		PORT MAP (
-- from component name 	=> to signal or port name
			E	=> G(8),
			D 	=> G(7),
			C	=> G(6),
			B 	=> G(4),
			A	=> G(3),
			Y	=> Q(7 downto 2)
		);

	Chip12	: DM74185
		PORT MAP (
-- from component name 		=> to signal or port name
			E				=> G(2),
			D 				=> G(1),
			C				=> G(0),
			B 				=> B(2),
			A				=> B(1),
			Y(6 downto 5)	=> Q(1 downto 0),
			Y(4)			=> T(0),
			Y(3 downto 1) 	=> U(3 downto 1)		
		);

	Chip13	: DM74185
		PORT MAP (
-- from component name 		=> to signal or port name
			E				=> G(17),
			D 				=> G(16),
			C				=> G(15),
			B 				=> Q(12),
			A				=> Q(11),
			Y(6 downto 4)	=> TTH(3 downto 1),
			Y(3 downto 1) 	=> J(6 downto 4)		
		);
		
	Chip14	: DM74185
		PORT MAP (
-- from component name 		=> to signal or port name
			E				=> Q(10),
			D 				=> Q(9),
			C				=> Q(8),
			B 				=> Q(6),
			A				=> Q(5),
			Y(6 downto 5)	=> J(3 downto 2),
			Y(4)			=> TH(0),			
			Y(3 downto 1) 	=> H(3 downto 1)		
		);

	Chip15: DM74185
		PORT MAP (
-- from component name 		=> to signal or port name
			E				=> '0',
			D 				=> Q(4),
			C				=> Q(3),
			B 				=> Q(2),
			A				=> Q(0),
			Y(6 downto 5)	=> J(1 downto 0),
			Y(4)			=> H(0),			
			Y(3 downto 1) 	=> T(3 downto 1)		
		);

	Chip16: DM74185
		PORT MAP (
-- from component name 		=> to signal or port name
			E				=> '0',
			D 				=> J(6),
			C				=> J(5),
			B 				=> J(4),
			A				=> J(2),
			Y(6 downto 5)	=> K(1 downto 0),
			Y(4)			=> TTH(0),			
			Y(3 downto 1) 	=> TH(3 downto 1)		
		);


-- Other circuits and equations:
U(0) <= B(0);
	
end structural_planC2;
						