--------------------------------------------------------------------------------
-- 4-bit carry generator circuit described using logic equations (structural). 
-- This circuit is used as a component for developing carry lookahead adders
-- https://digsys.upc.edu/csd/P03/Adder_4bit_CLA/Adder_4bit.html
---------------------------------------------------
-- CSD. P3 projects on arithmetic circuits 
-- http://digsys.upc.edu
--------------------------------------------------------------------------------

LIBRARY ieee;
USE  IEEE.STD_LOGIC_1164.all;

ENTITY Carry_generator IS
		PORT (
			Cin					: IN STD_LOGIC; 
			P0, P1, P2, P3		: IN STD_LOGIC; 
			G0, G1, G2, G3		: IN STD_LOGIC; 				
			Cout, C3, C2, C1	: OUT STD_LOGIC;
			G, P				: OUT STD_LOGIC
		  );
END Carry_generator;

ARCHITECTURE logic_equations_planA OF Carry_generator IS
BEGIN   

-- Equations translated from the tutorial:

C1 <= G0 or (P0 and Cin);
C2 <= G1 or (P1 and G0) or (P1 and P0 and Cin);
C3 <= G2 or (P2 and G1) or (P2 and P1 and G0) or (P2 and P1 and P0 and Cin) ;
Cout <= G3 or (P3 and G2) or (P3 and P2 and G1) or (P3 and P2 and P1 and G0) or (P3 and P2 and P1 and P0 and Cin) ; 

-- Group propagator and group generator outputs for chaining larger adder structures: 
G <= G3 or (P3 and G2) or (P3 and P2 and G1) or (P3 and P2 and P1 and G0); 
P <= P3 and P2 and P1 and P0;
				 		   	 		 		   	   
END logic_equations_planA;
