-------------------------------------------------------------------------------
-- An example circuit type DM74185 to be used as a chain building block 
-- VHDL plan B (behavioural, high-level, algorithmic) design 

-- File adapted from HEX_7SEG_decoder tutorial Plan B in P2
-- Translation from sketch diagram in the project specifications (Fig. 2):
-- https://digsys.upc.edu/csd/P02/Bin_BCD_6bitB/bin_BCD_6bit.html#DM74185  
---------------------------------------------------
-- Exercise P2  - CSD - Design of standard circuits
-- http://digsys.upc.edu
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------

LIBRARY ieee;
USE  IEEE.STD_LOGIC_1164.all;

entity DM74185 is
	port (
		E, D, C, B, A 	: 	in STD_LOGIC;
		Y				: 	out STD_LOGIC_VECTOR(6 DOWNTO 1)
	);
end DM74185;
	
architecture Truth_table_description of DM74185 is
	SIGNAL	Xin	: STD_LOGIC_VECTOR (4 DOWNTO 0);
BEGIN
	PROCESS (Xin)  
		BEGIN
				CASE Xin IS    		
					WHEN "00000" =>       		-- 0, 1
						Y <= "000000";
					WHEN "00001" =>       		-- 2, 3
						Y <= "000001";
					WHEN "00010" =>       		-- 4, 5
						Y <= "000010";
					WHEN "00011" =>       		-- 6, 7
						Y <= "000011";
					WHEN "00100" =>       		-- 8, 9
						Y <= "000100";
						
					WHEN "00101" =>       		-- 10, 11
						Y <= "001000";
					WHEN "00110" =>       		-- 12, 13
						Y <= "001001";
					WHEN "00111" =>       		-- 14, 15
						Y <= "001010";
					WHEN "01000" =>       		-- 16, 17
						Y <= "001011";
					WHEN "01001" =>       		-- 18, 19
						Y <= "001100";

					WHEN "01010" =>       		-- 20, 21
						Y <= "010000";
					WHEN "01011" =>       		-- 22, 23
						Y <= "010001";
					WHEN "01100" =>       		-- 24, 25
						Y <= "010010";
					WHEN "01101" =>       		-- 26, 27
						Y <= "010011";
					WHEN "01110" =>       		-- 28, 29
						Y <= "010100";
						
					WHEN "01111" =>       		-- 30, 31
						Y <= "011000";
					WHEN "10000" =>       		-- 32, 33
						Y <= "011001";
					WHEN "10001" =>       		-- 34, 35
						Y <= "011010";
					WHEN "10010" =>       		-- 36, 37
						Y <= "011011";
					WHEN "10011" =>       		-- 38, 39
						Y <= "011100";

					WHEN "10100" =>       		-- 40, 41
						Y <= "100000";
					WHEN "10101" =>       		-- 42, 43
						Y <= "100001";
					WHEN "10110" =>       		-- 44, 45
						Y <= "100010";
					WHEN "10111" =>       		-- 46, 47
						Y <= "100011";
					WHEN "11000" =>       		-- 48, 49
						Y <= "100100";
						
					WHEN "11001" =>       		-- 50, 51
						Y <= "101000";
					WHEN "11010" =>       		-- 52, 53
						Y <= "101001";
					WHEN "11011" =>       		-- 54, 55
						Y <= "101010";
					WHEN "11100" =>       		-- 56, 57
						Y <= "101011";
					WHEN "11101" =>       		-- 58, 59
						Y <= "101100";
						
					WHEN "11110" =>       		-- 60, 61
						Y <= "110000";
					WHEN Others =>       		-- 62, 63
						Y <= "110001";

				END CASE;
			
	END PROCESS;
			
-- Extra logic (simple buffers in this case)
Xin(0) <= A; 
Xin(1) <= B; 
Xin(2) <= C; 
Xin(3) <= D; 
Xin(4) <= E; 
	 		 							
end Truth_table_description;
								
