--------------------------------------------------------------------------------
-- Standard circuit HEX_to_7SEGMENTS decoder. Structural plan A. 
-- The outputs are minimised equations in the form of SoP generated by minilog
-- https://digsys.upc.edu/csd/P02/P2.html
---------------------------------------------------
-- Project P2  - CSD :  standard logic circuits in VHDL
-- http://digsys.upc.edu
--------------------------------------------------------------------------------

LIBRARY ieee;
USE  IEEE.STD_LOGIC_1164.all;

ENTITY HEX_7SEG_DECODER IS
	PORT (
		X_IN			        			: IN STD_LOGIC_VECTOR(3 DOWNTO 0); 
		BI_L                   	    		: IN STD_LOGIC;      
		A_L, B_L, C_L, D_L, E_L, F_L, G_L  	: OUT STD_LOGIC
		);
END HEX_7SEG_DECODER;            
 
ARCHITECTURE logic_equations OF HEX_7SEG_DECODER IS
BEGIN

	A_L <=	(NOT(X_in(3)) AND X_in(2) AND NOT(X_in(1)) AND NOT(X_in(0)) )	OR 
			( X_in(3)AND NOT(X_in(2)) AND X_in(1) AND X_in(0))				OR 
			( NOT(X_in(3))AND NOT(X_in(2)) AND NOT(X_in(1)) AND X_in(0) )	OR
	 		( X_in(3) AND X_in(2) AND NOT(X_in(1)) AND X_in(0) )			OR
	  		(NOT(BI_L));

	B_L <= 	(NOT(BI_L)) 													OR
	 		( NOT(X_in(3)) AND X_in(2) AND NOT(X_in(1)) AND X_in(0)) 		OR
	  		( X_in(3) AND X_in(1) AND X_in(0)) 								OR 
	  		( X_in(3) AND X_in(2) AND NOT(X_in(0))) 						OR
	   		( X_in(2) AND X_in(1) AND NOT(X_in(0)) );

	C_L <= 	(NOT(BI_L)) 													OR
			( X_in(3) AND X_in(2) AND NOT(X_in(0)) ) 						OR
			( NOT(X_in(3)) AND NOT(X_in(2)) AND X_in(1) AND NOT(X_in(0)))	OR
			( X_in(3) AND X_in(2) AND X_in(1) ) ;

	D_L <= 	(NOT(X_in(3)) AND X_in(2) AND NOT(X_in(1)) AND NOT(X_in(0))) 	OR
			( NOT(BI_L)) 													OR 
			( X_in(3) AND NOT(X_in(2)) AND X_in(1) AND NOT(X_in(0))) 		OR 
			( NOT(X_in(2)) AND NOT(X_in(1)) AND X_in(0)) 					OR 
			( X_in(2) AND X_in(1) AND X_in(0) );

	E_L <= 	( NOT(BI_L)) 									OR 	
			( NOT(X_in(2)) AND NOT(X_in(1)) AND X_in(0)) 	OR 
			( NOT(X_in(3)) AND X_in(2)AND NOT(X_in(1))) 	OR
			( NOT(X_in(3)) AND X_in(0) );

	F_L <= 	( X_in(3) AND X_in(2)AND NOT(X_in(1))AND X_in(0)) 	OR
			( NOT(BI_L)) 										OR 
			( NOT(X_in(3)) AND NOT(X_in(2))AND X_in(1)) 		OR
			( NOT(X_in(3))AND NOT(X_in(2)) AND X_in(0)) 		OR 
			( NOT(X_in(3))AND X_in(1) AND X_in(0) );

	G_L <= 	(NOT(BI_L) ) 											OR
			( X_in(3) AND X_in(2)AND NOT(X_in(1)) AND NOT(X_in(0))) OR 
			( NOT(X_in(3)) AND X_in(2) AND X_in(1) AND X_in(0)) 	OR
			(NOT(X_in(3)) AND NOT(X_in(2)) AND NOT(X_in(1)) );

 		   	 		 		   	   
END logic_equations;