--------------------------------------------------------------------------------
-- UPC - EETAC - CSD 
-- P4 example arithmetic circuit (optional extra material) 
-- 9-bit multiplier for integer numbers, signed, in two's complement (2C)
-- Plan C2 design plan. 
-- Lecture reference and schematics (see the Proteus circuit):
-- https://digsys.upc.edu/csd/P04/L04_1/L4_1.html
--------------------------------------------------------------------------------
LIBRARY ieee;
USE  IEEE.STD_LOGIC_1164.all;

entity Int_Mult_9bit is
   port
   (
      A, B:	in	std_logic_vector(8 downto 0);
      P, Q:	out	std_logic_vector(17 downto 0)
   );
end entity Int_Mult_9bit;
 
architecture hierarchical_plan_C2 of Int_Mult_9bit is

-- The component to be used:
COMPONENT Mult_9bit IS
	PORT (
		A, B	: IN STD_LOGIC_VECTOR(8 downto 0); 
		P		: OUT STD_LOGIC_VECTOR(17 downto 0)
	  );
END COMPONENT;

COMPONENT Two_C_9bit IS
	PORT (
		A		: IN 	STD_LOGIC_VECTOR(8 DOWNTO 0); 				
		B		: OUT 	STD_LOGIC_VECTOR (8 DOWNTO 0)
	  );
END COMPONENT;

COMPONENT Two_C_18bit IS
	PORT (
		A		: IN 	STD_LOGIC_VECTOR(17 DOWNTO 0); 
		Neg 	: IN 	STD_LOGIC;	
		B		: OUT 	STD_LOGIC_VECTOR (17 DOWNTO 0)
	  );
END COMPONENT;

-- Internal signals
	SIGNAL	C, D	: STD_LOGIC_VECTOR(8 DOWNTO 0); 	
	SIGNAL	R		: STD_LOGIC_VECTOR(17 DOWNTO 0); 	
	SIGNAL	Neg		: STD_LOGIC;

-- Instantiating the components in the circuit schematic
BEGIN
chip3	: Mult_9bit
	PORT MAP (
-- from component	=> to signal or port name
		A			=> C,			
		B			=> D,
		P			=> R
	);  

chip1	: Two_C_9bit
	PORT MAP (
-- from component	=> to signal or port name
		A			=> A,			
		B			=> C
	);  

chip2	: Two_C_9bit
	PORT MAP (
-- from component	=> to signal or port name
		A			=> B,			
		B			=> D
	);  

chip4	: Two_C_18bit
	PORT MAP (
-- from component	=> to signal or port name
		A			=> R,
		Neg 		=> Neg,
		B			=> P
	);  

-- Additional gates

Q <= R; 
Neg <= A(8) xor B(8);
			
		 
end architecture hierarchical_plan_C2;

